/*----------------------------------------------------------------------*/
/**
* File: conn_mul.c
*
* This file contains connection functions for multple sockets in server
* or client mode (hardware independent):
*   TCP_ConnectMul    = Connect Multiple Client Sockets
*   TCP_AcceptMul     = Accept Multiple Connection Requests
*   TCP_DisconnectMul = Disconnect Multiple Sockets
*/
/*----------------------------------------------------------------------*/

#ifdef _TI_CAMERA
  #include <sysvar.h>
  #include <vcrt.h>
#else
  #include <stdio.h>
#endif

#include "tcp_lib.h"

/****** TCP_ConnectMul = Connect Multiple Client Sockets = 11.2004 ******/
/*----------------------------------------------------------------------*/
/**
* The function connects all sockets in the input socket buffer as clients
* to a remote TCP/IP server, specified by "ip_addr" and "port".
*
* WARNING:
* Several connection requests issued by this function on PC can be
* received on camera in different order. There should be some
* wait time (about 500 ms or more). Here the couple of functions
*   TCP_ConnectMul
*   TCP_AcceptMul
* use another mechanism to accept requests in the same order. Adter
* accepting a connection, TCP_AcceptMul sends one byte to the client.
* After each successful connection, this function waits to receive
* a byte before next connection request.
*
* Parameters:
*   sock_buf  = Input/output socket buffer (already reset)
*   sock_cnt  = Input number of "sock_buf" elements (# of sockets to connect)
*   ip_addr   = Input string with IP address (NULL specifies default VC20xxE
*               IP address "192.168.0.65")
*   port      = Input port
*   wait_time = Input timeout in ms
*
* Return code:
*   TCP_OK  : OK
*   Other   : Returned by called functions
*
* Created:
*   Plamen Kozhuharov: 10.11.2004
*/
/*----------------------------------------------------------------------*/
int TCP_ConnectMul (
    TCP_SOCK   *sock_buf,       // input/output socket buffer
    int         sock_cnt,       // input # of sock_buf elements
    char       *ip_addr,        // input string with server IP address (NULL=default)
    int         port,           // input server port
    int         wait_time )     // input wait time in ms
{

/* DDD */
#define TRACE_CONN_MULTIPLE  0
#define DUMP_SOCKETS         0
#define WAIT_AFT_CONNECT     0

    int  rc = 0;
    int  rc1;
    int  isock;
//    unsigned long time;

#if TRACE_CONN_MULTIPLE
    printf("-- Platform: %s --\n",PLATFORM);
#endif

    for(isock=0; isock<sock_cnt; isock++)
    {

/*............. Connect to server */
        rc1 = TCP_Connect(&sock_buf[isock],ip_addr,port,wait_time);
        if(rc1 != TCP_OK)       /* failed */
        {
#if TRACE_CONN_MULTIPLE
          printf("TCP_ConnectMul: isock=%d: TCP_Connect error = %d\n",isock,rc1);
#endif
          if(rc == 0) rc = rc1;   // save 1st error
        }
        else                    /* success */
        {
#if TRACE_CONN_MULTIPLE
          printf("TCP_ConnectMul: isock=%d: TCP_Connect OK\n",isock);
#endif
#if DUMP_SOCKETS
          printf("TCP_ConnectMul: %d: socket=0x%x\n",
                    isock,sock_buf[isock].socket);
#endif

/* Receive one byte after successful connection */
          rc1 = TCP_WaitByte(&sock_buf[isock],300);
          if(rc1 < 0 && rc == 0) rc = rc1;   // save 1st error
        }

#if WAIT_AFT_CONNECT
#if !defined(_TI_CAMERA)
      Sleep(1300);
#endif
#endif

    } /* for(isock=0; isock<sock_cnt; isock++) */

    return rc;
}

/****** TCP_AcceptMul = Accept Multiple Connection Requests = 11.04 *****/
/*----------------------------------------------------------------------*/
/**
* The function waits for connection requests from multiple remote clients
* and connects the sockets in the input/outpit buffer "sock_buf" as server
* sockets. The input listen socket "listen_sock" must be initilaized by
* the calling function by TCP_Listen(). The input/outpit buffer "sock_buf"
* must be previously reset by the calling function by.
*
* WARNING:
* Several connection requests issued by remote PC client(s) can be
* received on camera in different order. There should be some
* wait time (about 500 ms or more). Here the two functions
*   TCP_ConnectMul
*   TCP_AcceptMul
* use another mechanism to accept requests in the same order. After
* accepting a connection, TCP_AcceptMul sends one byte to the client.
* After each successful connection of current client socket,
* TCP_ConnectMul waits to receive a byte before next connection request.
*
* Parameters:
*   listen_sock = Input listen socket (already initialized)
*   sock_buf    = Input/output socket buffer (already reset)
*   sock_cnt    = Input number of "sock_buf" elements (# of sockets to connect)
*   wait_time   = Input timeout in ms
*
* Return code:
*   TCP_OK                  : OK
*   TCP_NO_CONNECTION_ERROR : Sockets connection timeout
*   Other                   : Returned by called functions
*
* Created:
*   Plamen Kozhuharov: 10.11.2004
*/
/*----------------------------------------------------------------------*/
int TCP_AcceptMul (
    TCP_SOCK   *listen_sock,    // input listen socket (initialized)
    TCP_SOCK   *sock_buf,       // input/output socket buffer
    int         sock_cnt,       // input # of sock_buf elements
    int         wait_time )     // input wait time in ms
{

/* DDD */
#define TRACE_WAIT_CONN_MULTIPLE  0
#define DUMP_SERV_SOCKETS         0

    int  rc = 0;
    int  rc1 = 0;
    int  isock;
    unsigned long time;

#if DUMP_SERV_SOCKETS
#if defined(_TI_CAMERA)
    int iret;
    sockaddr_in pname;
    uint_16 namelen = sizeof(sockaddr_in);
#endif
#endif


#if TRACE_WAIT_CONN_MULTIPLE
    printf("-- Platform: %s --\n",PLATFORM);
#endif

    for(isock=0; isock<sock_cnt; isock++)
    {
      time = get_time();
      for(;;)
      {

        rc = TCP_AcceptConnection(listen_sock,&sock_buf[isock],1);
        if(rc != TCP_OK)
        {
#if TRACE_WAIT_CONN_MULTIPLE
  printf("TCP_WaitConnectMul: isock=%d: TCP_AcceptConnection error=%d\n",
                      isock,rc);
#endif
          goto done;
        }

        if(TCP_GetConnectionState(&sock_buf[isock]))    /* success */
        {
#if TRACE_WAIT_CONN_MULTIPLE
          printf("TCP_WaitConnectMul: isock=%d: Connect OK\n",isock);
#endif

#if DUMP_SERV_SOCKETS   /* [[[ */
#if defined(_TI_CAMERA)
          iret = getpeername(sock_buf[isock].socket,&pname,&namelen);
          if(iret == VCRT_OK)
          {
            printf("TCP_WaitConnectMul: %d: socket=0x%x, remote port=%d\n",
                      isock,
                      sock_buf[isock].socket,
                      pname.sin_port);
          }
          else
          {
            printf("TCP_WaitConnectMul: getpeername error=%d\n",iret);
          }
#endif
#endif  /* ]]] */

/* Send one byte after successful connection */
          rc = TCP_SendByte(&sock_buf[isock],isock);
          if(rc) goto done;
          break;
        }

        if(get_time() - time >= (unsigned int)wait_time)
        {
#if TRACE_WAIT_CONN_MULTIPLE
          printf("TCP_WaitConnectMul: isock=%d: Timeout\n",isock);
#endif
          rc1 = TCP_NO_CONNECTION_ERROR;
          break;
        }

      } /* for(;;) */

    } /* for(isock=0; isock<sock_cnt; isock++) */

#if !defined(_TI_CAMERA)
    Sleep(150);
#endif

/*............. Exit */
done:
    if(rc == 0) rc = rc1;
    return rc;

}

/****** TCP_DisconnectMul = Disconnect Multiple Sockets = 11.2004 *******/
/*----------------------------------------------------------------------*/
/**
* The function disconnects multiple sockets.
*
* Parameters:
*   sock_buf = Input/output socket buffer
*   sock_cnt = Input number of "sock_buf" elements (# of sockets to
*              disconnect)
*
* Return code:
*   None
*
* Created:
*   Plamen Kozhuharov: 11.11.2004
*/
/*----------------------------------------------------------------------*/

void TCP_DisconnectMul (
    TCP_SOCK   *sock_buf,       // input/output socket buffer
    int         sock_cnt )      // input # of sock_buf elements
{
    int isock;
    for(isock=0; isock<sock_cnt; isock++)
    {
       TCP_Disconnect(&sock_buf[isock]);
    }
    return;
}
