/*----------------------------------------------------------------------*/
/*
* File: vdemo.c
*
* Client demo program for the stand-alone TCP I/O tool library, which
* communicates with the server VIMOS I/O program VDEMO.AEF. The code
* is portable - you can run it on PC and camera. On camera the progam
* displays the received image on the monitor.
*
* VIMOS should be started with the server I/O program before you start
* this demo program on camera or on PC (VIMOS should be started on the
* opposite platform). It is possible also to connect two cameras or two
* PCs - one running this program and the other - VIMOS.

* Usage: Start vdemo in a command prompt window after VIMOS has been
*        started. Press any key to stop.
*
* WARNING: Remember to change if necessary the server's IP address for
*          the opposite platform (where VIMOS runs), set by the macro:
*               SERVER_IP_ADDR
*
* Created:  Plamen Kozhuharov, 17.01.2005
*/
/*----------------------------------------------------------------------*/

#include <string.h>
#include <stdlib.h>

#if defined(_TI_CAMERA)
  #include <sysvar.h>
  #include <vcrt.h>
#else
  #include <conio.h>
  #include <stdio.h>
#endif

// #include "tcp_lib.h"
#include "tcp_io.h"

/* DDD - Diag macros */
#define DUMP_VDEMO_ERRORS      1    /* dump errors                      */
#define TRACE_VDEMO            1    /* trace vdemo execution            */
#define FLUSH_INTERFACE        0    /* flush interface bef & after I/O  */
#define DUMP_RECV_PTLIST       0    /* dump received point-list         */

/*............. Server parms */
#define SERVER_IP_ADDR  "192.168.0.65"  /* server IP address (camera)   */
#define SERVER_PORT     2000            /* server listen port           */

/*............. Tool execution flags */
#define EXEC_SEND_RESULT  1     /* execute "Send result" tool           */
#define EXEC_RECV_RESULT  1     /* execute "Receive result" tool        */
#define EXEC_SEND_PTLIST  1     /* execute "Send point-list" tool       */
#define EXEC_RECV_PTLIST  1     /* execute "Receive point-list" tool    */
#define EXEC_SEND_IMAGE   1     /* execute "Send image" tool            */
#define EXEC_RECV_IMAGE   1     /* execute "Receive image" tool         */
#define EXEC_SEND_STRING  1     /* execute "Send string" tool           */
#define EXEC_RECV_STRING  1     /* execute "Receive string" tool        */
#define EXEC_SEND_BINARY  0     /* execute "Send binary" tool           */
#define EXEC_RECV_BINARY  0     /* execute "Receive binary" tool        */

/*............. Timeouts */
#define WAIT_REPLY          3000  /* wait for receiver reply (ms), 0=no wait */
#define WAIT_BEF_LOOP_CYCLE    0  /* wait before each pass (ms), 0=no wait   */
#define WAIT_BEF_SEND          0  /* wait before send (ms), 0=no wait        */

#define CONN_TIME_OUT       7000    /* connection wait time in ms       */
#define RECV_RESULT_TIMEOUT 2000    /* recv result timeout (ms)         */
#define RECV_PTLIST_TIMEOUT 2000    /* recv ptlist timeout (ms)         */
#define RECV_IMAGE_TIMEOUT  2000    /* recv image timeout (ms)          */
#define RECV_STRING_TIMEOUT 2000    /* recv string timeout (ms)         */
#define RECV_BINARY_TIMEOUT 2000    /* recv binary timeout (ms)         */


/*............. Image dimensions */
#ifdef _TI_CAMERA
  #define MAX_PITCH  getvar(VPITCH)   // max memory pitch (768)
  #define MAX_DY     getvar(VWIDTH)   // max image height (480)
#else
  #define MAX_PITCH  1024   // max memory pitch
  #define MAX_DY     800    // max image height
#endif

#if defined(_TI_CAMERA)
  #define KBHIT()   (-1 != rbempty())
#else
  #define KBHIT()   _kbhit()
#endif

/*............. Global data - TCP sockets */
    TCP_SOCK ListenSock;    /* listen socket  */
    TCP_SOCK DataSock;      /* data socket    */

/*............. Local functions */
int  exec_io ();
void fill_img (
    IMAGE  *img,                /* input/output image                   */
    int     seed );             /* input data seed                      */
void disp_img (
    IMAGE  *img,                /* input image                          */
    int     x,                  /* input x-coord. of top left screen corner */
    int     y );                /* input y-coord. of top left screen corner */


/****** main = Main Demo Function = 10.2004 *****************************/
int main(int argc, char *argv[])
{
    int  rc;
    char ip_str[32] = SERVER_IP_ADDR;
//    unsigned long time;

/*............. Get new IP address */
    if(argc >= 2)
    {
      strcpy(ip_str,argv[1]);      // save IP address
    }

/*----------------------------------------------------------------------*/
/*                                Setup                                 */
/*----------------------------------------------------------------------*/

/*............. Init TCP */
    rc = TCP_Init();
    if(rc)
    {
#if DUMP_VDEMO_ERRORS
       printf("VDEMO: TCP_Init error = %d\n",rc);
#endif
       return rc;
    }

/*............. Reset socket(s) */
    TCP_DataSockReset(&DataSock,1);

/*----------------------------------------------------------------------*/
/*                           Setup TCP client                           */
/*----------------------------------------------------------------------*/
/* NOTE: The server program should be already started ! */

/*............. Connect as client */
    rc = TCP_Connect(&DataSock,ip_str,SERVER_PORT,CONN_TIME_OUT);
    if(rc != TCP_OK)
    {
       printf("VDEMO: TCP_Connect error=%d\n",rc);
       goto done;
    }
    printf("VDEMO: Connected as client OK\n");

/*----------------------------------------------------------------------*/
/*                                 Do I/O                               */
/*----------------------------------------------------------------------*/

/*............. Flush interface before I/O */
#if FLUSH_INTERFACE
{
  int  iret;
  printf("--- Flush before I/O ----\n");
  while((iret = TCP_WaitByte(&DataSock, 5)) >= 0)
  {
    printf("0x%02x ",iret);
  }
  printf("\n");
}
#endif

/*............. Start I/O */
    rc = exec_io();

/*............. Exit */
done:

/*............. Flush interface after I/O */
#if FLUSH_INTERFACE
{
  int  iret;
  printf("--- Flush after I/O ----\n");
  while((iret = TCP_WaitByte(&DataSock, 5)) >= 0)
  {
    printf("0x%02x ",iret);
  }
  printf("\n");
}
#endif

/*............. Wait before disconnect */
#ifdef _TI_CAMERA
    time_delay(500);
#else
    Sleep(500);
#endif

/*............. Disconnect sockets */
    TCP_Disconnect(&DataSock);
    TCP_Close();

#if TRACE_VDEMO
    printf("VDEMO: RC=%d\n",rc);
#endif
    return rc;

}

/****** exec_io = Execute TCP I/O tools = 11.2004 ***********************/
/*----------------------------------------------------------------------*/
/*
* This function communicates with other TCP I/O tools or VIMOS TCP tools.
* The function executes the following tool sequence:
*       "Send result"
*       "Receive result"
*       "Send point-list"
*       "Receive point-list"
*       "Send image"
*       "Receive image"
*        . . . . . . . . . .
*
* Parameters:
*   None
*
* Return code:
*   0     = OK
*   Other = Errors
*
* Created:  Plamen Kozhuharov, 09.11.2004
*/
/*----------------------------------------------------------------------*/
int  exec_io ()
{

/* DDD */
#define TRACE_EXEC_IO  1

    int      rc = TCP_OK;
    unsigned char *ptl_buf = NULL;
    unsigned char *img_buf = NULL;

    int      ptl_size = 100000;
    int      img_size = MAX_PITCH*MAX_DY + 256;

    IMAGE    img;
    int      iter;

#ifdef _TI_CAMERA
    unsigned long time;
#endif
    unsigned long time1;

#define RES_TYPE   0    /* 0=float, 1=point, 2=int16, 3=int32 */
#if EXEC_SEND_RESULT
    void     *res_ptr;              // ptr to result buffer
    int       res_format = 1;       // binary format only !
#if RES_TYPE == 0
    float     res_f;                // float result
#elif RES_TYPE == 1
    PTL_POINT res_p;                // point result
#elif RES_TYPE == 2
    TCP_INT16 res_int16;            // 16-bit integer result
#else
    TCP_INT32 res_int32;            // 32-bit integer result
#endif
#endif

#if EXEC_RECV_RESULT
    char      res_buf[8];           // receive result buffer
    int       res_size;             // output result size
    int       res_type;             // output result type
    PTL_POINT res_pt;               // output point
#endif

#if EXEC_SEND_PTLIST
    int      ptl_len;
    PTL_ITEM ptl_item;
#endif

#if EXEC_RECV_PTLIST
    int      ptl_len1;
#endif

#if EXEC_RECV_IMAGE
    int      img_type;
    int      img_qual;
#endif

#if EXEC_SEND_STRING
#define STR_SIZE 100
    char    str_buf[STR_SIZE+1];  // string buffer
    int     str_size;
#endif

#if EXEC_SEND_BINARY
#define BIN_SIZE 100
    char    bin_buf[BIN_SIZE+1];  // binarybuffer
    int     bin_size;
#endif

#if DUMP_RECV_PTLIST
    PTL_ITEM *pit;
    int       i;
#endif


/*----------------------------------------------------------------------*/
/*                                Setup                                 */
/*----------------------------------------------------------------------*/
    rc = TCP_NO_MEMORY;

/*............. Allocate point-list buffer */
    ptl_buf = (unsigned char *)tcp_alloc(ptl_size);
    if(ptl_buf == NULL)
    {
#if DUMP_VDEMO_ERRORS
      printf("VDEMO: Unable to allocate point-list buffer\n");
#endif
      goto done;
    }

/*............. Allocate image buffer */
    img_buf = (unsigned char *)tcp_alloc(img_size);
    if(img_buf == NULL)
    {
#if DUMP_VDEMO_ERRORS
      printf("VDEMO: Unable to allocate image buffer\n");
#endif
      goto done;
    }

    rc = TCP_OK;


/*----------------------------------------------------------------------*/
/*                 Loop: Execute send and receive tools                 */
/*----------------------------------------------------------------------*/
/*
* Each loop cycle executes all enabled by macros I/O tools.
*/

/*............. Loop setup */
    img.img_buf = (char *)img_buf;  /* save ptr to image buffer         */
    iter = 0;               /* iteration count */

/*............. Loop */
    for(;;)
    {

#if TRACE_EXEC_IO
  printf("--- iter=%d ---\n",iter);
#endif

/*............. Exit loop on any pressed key */
      if(KBHIT())
      {
#if TRACE_EXEC_IO
        printf("Intrpt by user\n");
#endif
        break;
      }

#ifdef _TI_CAMERA
      tpict();
//      vmode(4);
#endif

/*............. Wait before next loop cycle */
/* To slow down message dump */
#if WAIT_BEF_LOOP_CYCLE
      time = get_time();
      while(get_time() - time < WAIT_BEF_LOOP_CYCLE);
#endif

/*----------------------------------------------------------------------*/
/*                               Send result                            */
/*----------------------------------------------------------------------*/
#if EXEC_SEND_RESULT  /* [[[ */

/*............. Wait before next send tool */
#if WAIT_BEF_SEND
      time = get_time();
      while(get_time() - time < WAIT_BEF_SEND);
#endif

/*............. Init result buffer */
#if RES_TYPE == 0       /* float    */
      res_f = (float)3.14;
      res_ptr = &res_f;
#endif
#if RES_TYPE == 1       /* point    */
      res_p.x = 100;
      res_p.y = 200;
      res_p.subx = 1;
      res_p.suby = 2;
      res_ptr = &res_p;
#endif
#if RES_TYPE == 2       /* angle    */
      res_int16 = 123;
      res_ptr = &res_int16;
#endif
      res_format = 1;   /* binary format */

/*............. Send result */
      time1 = get_time();
      rc = TCP_SendResult(&DataSock,res_ptr,RES_TYPE,res_format,WAIT_REPLY);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Send result error = %d\n",rc);
#endif
         break;
      }
      printf("VDEMO: Send result OK, time=%d\n",(int)time1);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                            Receive result                            */
/*----------------------------------------------------------------------*/
#if EXEC_RECV_RESULT  /* [[[ */
      res_size = 8;

      time1 = get_time();
      rc = TCP_RecvResult(&DataSock,res_buf,&res_size,&res_type,
                            RECV_RESULT_TIMEOUT);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
          printf("VDEMO: Receive result error = %d\n",rc);
#endif
          goto done;
      }

/*............. Success */
      printf("VDEMO: Receive result OK: type=%d size=%d time=%d\n",
                res_type,res_size,(int)time1);
      if(res_type == 0)
      {
         printf("        Float result = %f\n",*(float *)res_buf);
      }
      else if(res_type == 1)
      {
         memcpy(&res_pt,res_buf,sizeof(PTL_POINT));
         printf("        Point result = (%d.%d, %d.%d)\n",
                   res_pt.x, res_pt.subx,
                   res_pt.y, res_pt.suby);
      }

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                             Send point-list                          */
/*----------------------------------------------------------------------*/
#if EXEC_SEND_PTLIST  /* [[[ */

/*............. Wait before next send tool */
#if WAIT_BEF_SEND
      time = get_time();
      while(get_time() - time < WAIT_BEF_SEND);
#endif

/*............. Init point-list buffer */
      ptl_len = 3;                    /* length of sent point-list        */
      memset(&ptl_item,0,sizeof(PTL_ITEM));

      ptl_item.pt.x = 100;
      ptl_item.pt.subx = 1;
      ptl_item.pt.y = 200;
      ptl_item.pt.suby = 2;
      ptl_item.fpar[0] = 10;
      ptl_item.fpar[1] = 11;

      memcpy(ptl_buf,&ptl_item,sizeof(PTL_ITEM));

      ptl_item.pt.x = 101;
      ptl_item.pt.subx = 3;
      ptl_item.pt.y = 201;
      ptl_item.pt.suby = 4;
      ptl_item.fpar[0] = 20;
      ptl_item.fpar[1] = 21;

      memcpy(ptl_buf+sizeof(PTL_ITEM),&ptl_item,sizeof(PTL_ITEM));
      memcpy(ptl_buf+2*sizeof(PTL_ITEM),&ptl_item,sizeof(PTL_ITEM));

/*............. Send point-list */
      time1 = get_time();
      rc = TCP_SendPtlist(&DataSock,ptl_buf,ptl_len,WAIT_REPLY);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Send point-list error = %d\n",rc);
#endif
         break;
      }
      printf("VDEMO: Send point-list OK, time=%d\n",(int)time1);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                          Receive point-list                          */
/*----------------------------------------------------------------------*/
#if EXEC_RECV_PTLIST  /* [[[ */
      ptl_len1 = ptl_size/sizeof(PTL_ITEM);   /* max # in items */

      time1 = get_time();
      rc = TCP_RecvPtlist(&DataSock,ptl_buf,&ptl_len1,RECV_PTLIST_TIMEOUT);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
           printf("VDEMO: Receive point-list error = %d\n",rc);
#endif
           goto done;
      }

      printf("VDEMO: Receive point-list OK: len=%d time=%d\n",
                ptl_len1,(int)time1);

/*............. Dump received point-list */
#if DUMP_RECV_PTLIST
  pit = (PTL_ITEM *)ptl_buf;
  for(i=0; i<ptl_len1; i++)
  {
    printf("%d: (%3d,%3d) %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f\n",
             i,
             pit->pt.x,pit->pt.y,
             pit->fpar[0],pit->fpar[1],pit->fpar[2],pit->fpar[3],
             pit->fpar[4],pit->fpar[5],pit->fpar[6],pit->fpar[7]);
    pit++;
  }
#endif

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                               Send image                             */
/*----------------------------------------------------------------------*/
#if EXEC_SEND_IMAGE   /* [[[ */

/*............. Wait before next send tool */
#if WAIT_BEF_SEND
      time = get_time();
      while(get_time() - time < WAIT_BEF_SEND);
#endif

/*............. Init image */
/*
      img.dx = 640;
      img.dy = 480;
      img.pitch = 768;
*/
      img.dx = 300;
      img.dy = 200;
      img.pitch = 512; //172;

      fill_img(&img,iter);       /* fill image with data */

/*............. Send image */
      time1 = get_time();
      rc = TCP_SendImage(&DataSock,&img,0,0,WAIT_REPLY);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Send image error = %d\n",rc);
#endif
         break;
      }
      printf("VDEMO: Send image OK, time=%d\n",(int)time1);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                            Receive image                             */
/*----------------------------------------------------------------------*/
#if EXEC_RECV_IMAGE   /* [[[ */

      time1 = get_time();
      rc = TCP_RecvImage(&DataSock,&img,img_size,RECV_IMAGE_TIMEOUT,
                                &img_type,&img_qual);
      time1 = get_time() - time1;
      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Receive image error = %d\n",rc);
#endif
         goto done;
      }

      printf("VDEMO: Receive image OK: dx,dy=%d,%d time=%d\n",
                       img.dx,img.dy,(int)time1);

#ifdef _TI_CAMERA
      disp_img(&img,250,250);
      time = get_time();    // wait a little
      while(get_time() - time < 500);
#endif

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                               Send string                            */
/*----------------------------------------------------------------------*/
#if EXEC_SEND_STRING  /* [[[ */

/*............. Wait before next send tool */
#if WAIT_BEF_SEND
      time = get_time();
      while(get_time() - time < WAIT_BEF_SEND);
#endif

/*............. Init string buffer */
      memset(str_buf,'1',STR_SIZE);
      memcpy(str_buf,"Ala-bala-portokala..",20);
      str_size = STR_SIZE;

/*............. Send string */
      time1 = get_time();
      rc = TCP_SendString(&DataSock,str_buf,str_size,WAIT_REPLY);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Send string error = %d\n",rc);
#endif
         break;
      }

/*............. Success */
      printf("VDEMO: Send string OK, time=%d\n",(int)time1);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                            Receive string                            */
/*----------------------------------------------------------------------*/
#if EXEC_RECV_STRING  /* [[[ */
      str_size = STR_SIZE;

      time1 = get_time();
      rc = TCP_RecvString(&DataSock,str_buf,&str_size,RECV_STRING_TIMEOUT);
      time1 = get_time() - time1;

      if(rc != TCP_OK)
      {
#if DUMP_VDEMO_ERRORS
          printf("VDEMO: Receive string error = %d\n",rc);
#endif
          goto done;
      }

/*............. Success */
      printf("VDEMO: Receive string OK: len=%d time=%d\n",
                str_size,(int)time1);
      str_buf[str_size] = 0;
      printf("        str=%s\n",str_buf);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                              Send binary                             */
/*----------------------------------------------------------------------*/
#if EXEC_SEND_BINARY  /* [[[ */

/*............. Wait before next send tool */
#if WAIT_BEF_SEND
      time = get_time();
      while(get_time() - time < WAIT_BEF_SEND);
#endif

/*............. Init binary buffer */
      memset(bin_buf,222,BIN_SIZE);
      bin_size = 1;

/*............. Send binary */
      time1 = get_time();
      rc = TCP_SendBinary(&DataSock,(unsigned char *)bin_buf,bin_size);
      time1 = get_time() - time1;

      if(rc != bin_size)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Send binary error: E=%d A=%d\n",bin_size,rc);
#endif
         break;
      }

/*............. Success */
      printf("VDEMO: Send binary OK, time=%d\n",(int)time1);

#endif  /* ]]] */

/*----------------------------------------------------------------------*/
/*                            Receive binary                            */
/*----------------------------------------------------------------------*/
#if EXEC_RECV_BINARY  /* [[[ */
      bin_size = 4;     /* sync with VDEMO.AEF "Send binary" tool */

      time1 = get_time();
      rc = TCP_RecvBinary(&DataSock,(unsigned char *)bin_buf,bin_size,
                          RECV_BINARY_TIMEOUT);
      time1 = get_time() - time1;

      if(rc != bin_size)
      {
#if DUMP_VDEMO_ERRORS
         printf("VDEMO: Receive binary error: E=%d A=%d\n",bin_size,rc);
#endif
          goto done;
      }

/*............. Success */
      printf("VDEMO: Receive binary OK: len=%d time=%d\n",
                bin_size,(int)time1);
      bin_buf[bin_size] = 0;
      printf("        BIN=%s\n",bin_buf);

#endif  /* ]]] */


      iter++;
//      if(iter > 10) break;

    } /* for(;;) : main communication loop */


/*............. Exit */
done:
#ifndef _TI_CAMERA
//    TCP_ReadEcho(sock_23,1,CAM_ECHO_TIMEOUT);    // dump camera diag messages (if any)
#endif

    if(ptl_buf != NULL) tcp_free(ptl_buf);
    if(img_buf != NULL) tcp_free(img_buf);

    return rc;
}

/****** fill_img = Fill image with data = 11.2004 ***********************/
void fill_img (
    IMAGE  *img,                /* input/output image                   */
    int     seed )              /* input data seed                      */
{
    int i;
    char *ptr = img->img_buf;

#ifdef _TI_CAMERA
    int addr = getvar(CAPT_START);
    seed = seed;        // keep compiler happy
#endif

    seed &= 0x000f;

    for(i=0; i<img->dy; i++)
    {
#ifdef _TI_CAMERA
       memcpy(ptr,(char *)addr,img->dx);
       addr += getvar(VPITCH);
#else
       memset(ptr,(255-i*seed) & 0xff,img->dx);
#endif

       ptr += img->pitch;
    }
}

/****** disp_img = Display image = 11.2004 ******************************/
void disp_img (
    IMAGE  *img,                /* input image                          */
    int     x,                  /* input x-coord. of top left screen corner */
    int     y )                 /* input y-coord. of top left screen corner */
{

#ifdef _TI_CAMERA
    int i;
    char *ptr = img->img_buf;
    int addr = getvar(DISP_START) + y*getvar(VPITCH) + x;

    vmode(1);   // set still mode
    for(i=0; i<img->dy; i++)
    {
       memcpy((char *)addr,ptr,img->dx);

       addr += getvar(VPITCH);
       ptr += img->pitch;
    }
#endif
    return;
}

